use vars qw(@t $ff $ss $folderfirst $sortstate);
$folderfirst = param('folderfirst') ? 1 : 0;
$ss          = param('sort')        ? 1 : 0;
folderFirst($folderfirst);
sub ShowBookmarks
{
    loadTree($m_hrSettings->{tree}{links});
    *t = \@{$HTML::Menu::TreeView::TreeView[0]};
    applyRights(\@t);
    print br().'<div class="showTables">';

    _showBookmarksNavi();
    print qq(<table class="ShowTables" style="padding-top:1.65em;" align="center"  border="0" cellpadding="0" cellspacing="0"  width="100%" summary="linkLayout"><tr><td valign="top">);
    print Tree(\@t);
    print qq(</td></tr></table></div>);
}

sub _showBookmarksNavi
{
    $ff = $folderfirst;
    $ff = $ff ? 0 : 1;
    sortTree($ss);
    $sortstate = $ss;
    $ss = $ss ? 0 : 1;
    print 
        '<div align="left">' 
      . br()
      . a(
          {
           class => 'link',
           href  => "javascript:requestURI('$ENV{SCRIPT_NAME}?action=links&sort=1','$m_sAction','$m_sTitle')",
           title => translate('test')
          },
          translate('sort')
      ) . ' | ';
    print a(
                     {
                      class => $folderfirst ? 'currentLink' : 'link',
                      href  => "javascript:requestURI('$ENV{SCRIPT_NAME}?action=links&sort=$sortstate&folderfirst=$ff','$m_sAction','$m_sTitle')",
                      title => translate('folderFirst')
                     },
                     translate('folderFirst')
    ) . ' | ';
    print a(
                     {
                      class => 'link',
                      href  => "javascript:requestURI('$ENV{SCRIPT_NAME}?action=ExportOperaBookmarks','$m_sAction','$m_sTitle')",
                      title => translate('ExportOperaBookmarks')
                     },
                     translate('ExportOperaBookmarks')
    );
    print ' | '
      . a(
          {
           class => 'link',
           href  => "javascript:requestURI('$ENV{SCRIPT_NAME}?action=editTreeview&dump=links','$m_sAction','$m_sTitle')",
           title => translate('edit')
          },
          translate('edit')
      ) if ($m_nRight >= $m_oDatabase->getActionRight('editTreeview'));
    print ' | '
      . a(
          {
           class => $m_sAction eq "ImportOperaBookmarks"
           ? 'currentLink'
           : 'link',
           href  => "javascript:requestURI('$ENV{SCRIPT_NAME}?action=ImportOperaBookmarks','$m_sAction','$m_sTitle')",
           title => translate("ImportOperaBookmarks")
          },
          translate("ImportOperaBookmarks")
      ) if ($m_nRight >= $m_oDatabase->getActionRight('ImportOperaBookmarks'));
    print ' | '
      . a(
          {
           class => $m_sAction eq "ImportFireFoxBookmarks"
           ? 'currentLink'
           : 'link',
           href  => "javascript:requestURI('$ENV{SCRIPT_NAME}?action=ImportFireFoxBookmarks','$m_sAction','$m_sTitle')",
           title => translate('importFireFox'),
           id    => 'ShareLink'
          },
          translate('importFireFox')
      ) if ($m_nRight >= $m_oDatabase->getActionRight('ImportFireFoxBookmarks'));


    $msContent .= q|</div>| . br();
}

sub ExportOperaBookmarks
{
    loadTree($m_hrSettings->{tree}{links});
    *t = \@{$HTML::Menu::TreeView::TreeView[0]};
#     my %parameter = (
#                      path   => $m_hrSettings->{cgi}{bin} . '/templates',
#                      style  => $m_sStyle,
#                      title  => " ",
#                      server => $m_hrSettings->{serverName},
#                      id     => 'ShowBookmarks',
#                      class  => 'min',
#     );
#     my $window = new HTML::Window(\%parameter);
#     print br() . $window->windowHeader();
    _showBookmarksNavi();
    print qq(<table align="center"  border="0" cellpadding="0" cellspacing="0"  width="100%" summary="linkLayout"><tr><td align="center" valign="top">);
    print qq(<textarea style="width:98%;height:800px;">\nOpera Hotlist version 2.0\nOptions: encoding = utf-8, version=3\n);
    &_rec(\@t);
    print qq(</textarea><br/>);
    print qq(</td></tr></table>);
#     print $window->windowFooter();
}

sub _rec
{
    my $tree = shift;
    for (my $i = 0; $i < @$tree; $i++) {
        if (defined @$tree[$i]) {
            my $text = defined @$tree[$i]->{text} ? @$tree[$i]->{text} : '';
            if (defined @{@$tree[$i]->{subtree}}) {
                print "#FOLDER\n\tID=@$tree[$i]->{rid}\n\tNAME=$text\n\tUNIQUEID=@$tree[$i]->{rid}\n";
                _rec(\@{@$tree[$i]->{subtree}});
                print "-\n\n";
            } else {
                my $hrf = defined @$tree[$i]->{href} ? @$tree[$i]->{href} : '';
                print "#URL\n\tID=@$tree[$i]->{rid}\n\tNAME=$text\n\tURL=$hrf\n\tUNIQUEID=@$tree[$i]->{rid}\n";
            }
        }
    }
}

sub ImportOperaBookmarks
{
    my $save        = translate('save');
    my $choosefile  = translate('choosefile');
    my $newFolder   = translate('newFolder');
    my $b_NewFolder = param('newFolder') ? param('newFolder') : '';
    print qq|
<br/><div align="center">
<font size="+1">Upload Opera Bookmarks</font><br/><br/>
<form name="upload" method="post" accept-charset="utf-8" accept="text/*" enctype="multipart/form-data" onSubmit="submitForm(this,'$m_sAction','$m_sTitle');return false;">
<input name="file" type="file" size ="30" title="$choosefile"/>
$newFolder<input type="checkbox" name="newFolder"/>
<input type="submit" value="$save"/>
<input  name="action" value="ImportOperaBookmarks" style="display:none;"/>
</form></div><br/>|;
    my $sra = 0;
    my $ufi = param('file');

    if ($ufi) {
        use vars qw(@adrFile $folderId $currentOpen @openFolders @operaTree $treeTempRef $up);
        $up = upload('file');
        while (<$up>) {push @adrFile, $_;}
        ($folderId, $currentOpen) = (0) x 2;

        if ($b_NewFolder eq 'on') {
            loadTree($m_hrSettings->{tree}{links});
            unshift @fireFoxTree, @{$HTML::Menu::TreeView::TreeView[0]};
        }
        $treeTempRef = \@operaTree;
        $openFolders[0][0] = $treeTempRef;
        for (my $line = 0; $line < $#adrFile; $line++) {
            chomp $adrFile[$line];
            if ($adrFile[$line] =~ /^#FOLDER/) {    #neuer Folder
                $folderId++;
                my $text = $1 if ($adrFile[$line + 1] =~ /NAME=(.*)$/);
                $text = $1 if ($adrFile[$line + 2] =~ /NAME=(.*)$/);
                push @{$treeTempRef},
                  {
                    text => $text =~ /(.{50}).+/ ? "$1..." : $text,
                    subtree => []
                  };
                my $l = @$treeTempRef;
                $treeTempRef               = \@{@{$treeTempRef}[$l - 1]->{subtree}};    #aktuelle referenz setzen.
                $openFolders[$folderId][0] = $treeTempRef;                              #referenz auf den parent Tree speichern
                $openFolders[$folderId][1] = $currentOpen;                              #rücksprung speichern
                $currentOpen               = $folderId;
            }
            if ($adrFile[$line] =~ /^-/) {                                              #wenn folder geschlossen wird
                $treeTempRef = $openFolders[$openFolders[$currentOpen][1]][0];          #aktuelle referenz auf parent referenz setzen
                $currentOpen = $openFolders[$currentOpen][1];                           #rücksprung zu parent
            }
            if ($adrFile[$line] =~ /^#URL/) {                                           #Node anhängen
                my $text = $1 if ($adrFile[$line + 2] =~ /NAME=(.*)$/);
                $text = $1 if ($adrFile[$line + 1] =~ /NAME=(.*)$/);
                my $href = $1 if ($adrFile[$line + 3] =~ /URL=(.*)$/);
                $href = $1 if ($adrFile[$line + 2] =~ /URL=(.*)$/);
                if (defined $text && defined $href) {
                    push @{$treeTempRef},
                      {
                        text => $text =~ /(.{75}).+/ ? "$1..." : $text,
                        href => $href,
                        target => "_blank",
                      };
                }
            }
        }
        saveTree($m_hrSettings->{tree}{links}, \@operaTree);
    }
    ShowBookmarks();
}

sub ImportFireFoxBookmarks
{
    my $save        = translate('save');
    my $choosefile  = translate('choosefile');
    my $newFolder   = translate('newFolder');
    my $b_NewFolder = param('newFolder') ? param('newFolder') : '';
    print qq|
<br/><div align="center">
<font size="+1">Upload Firefox Bookmarks</font><br/><br/>
<form name="upload" action="$ENV{SCRIPT_NAME}" method="post" accept-charset="utf-8" accept="text/*" enctype="multipart/form-data" onSubmit="submitForm(this,'$m_sAction','$m_sTitle');return false;">
<input name="file" type="file" size ="30" title="$choosefile"/>
$newFolder<input type="checkbox"  name="newFolder"/>
<input type="submit" value="$save"/>
<input  name="action" value="ImportFireFoxBookmarks" style="display:none;"/>
</form></div><br/>|;
    my $sra = 0;
    my $ufi = param('file');

    if ($ufi) {
        use vars qw(@adrFile $folderId $currentOpen @openFolders @fireFoxTree $treeTempRef $up);
        $up = upload('file');
        while (<$up>) {push @adrFile, $_;}
        ($folderId, $currentOpen) = (0) x 2;
        if ($b_NewFolder eq 'on') {
            loadTree($m_hrSettings->{tree}{links});
            unshift @fireFoxTree, @{$HTML::Menu::TreeView::TreeView[0]};
        }
        $treeTempRef       = \@fireFoxTree;
        $openFolders[0][0] = $treeTempRef;
        $openFolders[0][1] = 0;
        for (my $line = 0; $line < $#adrFile; $line++) {
            chomp $adrFile[$line];
            if ($adrFile[$line] =~ /<DL>/) {    #neuer Folder
                $folderId++;
                if ($adrFile[$line - 1] =~ /<H3[^>]+>(.*)<\/H3>/) {
                    my $text = $1;
                    push @{$treeTempRef},
                      {
                        text => $text =~ /(.{50}).+/ ? "$1..." : $text,
                        subtree => []
                      };
                    my $l = @$treeTempRef;
                    $treeTempRef               = \@{@{$treeTempRef}[$l - 1]->{subtree}};    #aktuelle referenz setzen.
                    $openFolders[$folderId][0] = $treeTempRef;                              #referenz auf den parent Tree speichern
                    $openFolders[$folderId][1] = $currentOpen;                              #rücksprung speichern
                    $currentOpen               = $folderId;
                }
            }
            if ($adrFile[$line] =~ /<\/DL>/) {                                              #wenn folder geschlossen wird
                $treeTempRef = $openFolders[$openFolders[$currentOpen][1]][0];              #aktuelle referenz auf parent referenz setzen
                $currentOpen = $openFolders[$currentOpen][1];                               #rücksprung zu parent
            }
            if ($adrFile[$line] =~ /<A HREF="([^"]+)"[^>]+>(.*)<\/A>/) {                    #Node anhängen
                my $text = $2;
                my $href = $1;
                if (defined $text && defined $href) {
                    push @{$treeTempRef},
                      {
                        text => $text =~ /(.{75}).+/ ? "$1..." : $text,
                        href => $href,
                        target => "_blank",
                      };
                }
            }
        }
        saveTree($m_hrSettings->{tree}{links}, \@fireFoxTree);
    }
    ShowBookmarks();
}
